@@ -13,7 +13,7 @@
   packageStartupMessage("**  - Matrix Reconstruction from a Few Entries")
   packageStartupMessage("**")
   packageStartupMessage("** Version    : ",this.version,"      (",this.year,")",sep="")
-  packageStartupMessage("** Maintainer : Kisung You (kyoustat@gmail.com)")
+  packageStartupMessage("** Maintainer : Kisung You (kisungyou@outlook.com)")
   packageStartupMessage("**")
   packageStartupMessage("** Please share any bugs or suggestions to the maintainer.")
   packageStartupMessage("**--------------------------------------------------------**")
